/*
 * Basic test of using the softdevice App timer to toggle an I/O pin 
 */


#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "app_error.h"
#include "nrf_gpio.h"
#include "nrf51_bitfields.h"
#include "ble.h"
#include "ble_hci.h"
#include "ble_srv_common.h"
#include "ble_advdata.h"
#include "ble_conn_params.h"
#include "app_scheduler.h"
#include "softdevice_handler.h"
#include "app_timer_appsh.h"
#include "app_gpiote.h"

#define LED_PIN_NUM 										10


#define APP_TIMER_PRESCALER             0                                          
#define APP_TIMER_MAX_TIMERS            4                                          
#define APP_TIMER_OP_QUEUE_SIZE         6                                          

//<DGH>  define a timer for handing blinking the LED
#define LED_BLINK_OFF_TIME							APP_TIMER_TICKS(600, APP_TIMER_PRESCALER)  
#define LED_BLINK_ON_TIME							  APP_TIMER_TICKS(200, APP_TIMER_PRESCALER)  
static app_timer_id_t 									m_led_blink_timer_id;
static uint8_t     											LED_state;


#define SCHED_MAX_EVENT_DATA_SIZE       sizeof(app_timer_event_t)                 
#define SCHED_QUEUE_SIZE                10                                        


static void led_blink_timer_handler(void * p_context)
{
    uint32_t err_code, next_timer_interval; 

		if(LED_state)
		{
			 nrf_gpio_pin_set(LED_PIN_NUM);  //turn off the LED
			 next_timer_interval = LED_BLINK_OFF_TIME;   //stay off for OFF_TIME 
		}
		else
		{
			 nrf_gpio_pin_clear(LED_PIN_NUM);  //turn on the LED
			 next_timer_interval = LED_BLINK_ON_TIME;   //turn on for ON_TIME 
		}
		LED_state = !LED_state;	
    
	  err_code = app_timer_start(m_led_blink_timer_id, next_timer_interval, NULL);  //reload the timer
    APP_ERROR_CHECK(err_code); 
}


static void gpio_init(void)
{
	  nrf_gpio_cfg_output(LED_PIN_NUM);
}


static void timers_init(void)
{
    uint32_t err_code;  
	  
    APP_TIMER_APPSH_INIT(APP_TIMER_PRESCALER, APP_TIMER_MAX_TIMERS, APP_TIMER_OP_QUEUE_SIZE, true);

	  LED_state = 0;
    err_code = app_timer_create(&m_led_blink_timer_id, APP_TIMER_MODE_SINGLE_SHOT, led_blink_timer_handler);
    APP_ERROR_CHECK(err_code);
}


static void timers_start(void)
{
    uint32_t err_code;
		
    err_code = app_timer_start(m_led_blink_timer_id, LED_BLINK_OFF_TIME, NULL);
    APP_ERROR_CHECK(err_code); 	
}


static void soft_device_init(void)
{
    SOFTDEVICE_HANDLER_INIT(NRF_CLOCK_LFCLKSRC_XTAL_20_PPM, NULL);
}


static void scheduler_init(void)
{
    APP_SCHED_INIT(SCHED_MAX_EVENT_DATA_SIZE, SCHED_QUEUE_SIZE);
}


static void power_manage(void)
{
    uint32_t err_code = sd_app_evt_wait();
    APP_ERROR_CHECK(err_code);
}


int main(void)
{
    gpio_init();
    timers_init();
	  soft_device_init();
	
		nrf_gpio_pin_set(LED_PIN_NUM);
  
    scheduler_init();
    timers_start();
	
  
  for (;;)
    {
        app_sched_execute();
        power_manage();
    }
}

